require 'redis'
require 'redis/namespace'

module MerryGoRound
  GRANULARITIES = [:minute, :hour, :day, :week, :year].freeze

  def self.configure
    yield self
  end

  def self.aggregate!
    Aggregator.new.aggregate!
  end

  def self.record(*args)
    Entry.new(*args).record!
  end

  def self.redis
    # Set redis to nothing make the setter run and setup a default if it's nothing
    self.redis = {} unless defined? @@redis

    # Return the namespaced Redis instance
    @@redis
  end

  def self.redis=(options = {})
    client = nil
    if options.is_a?(Redis)
      client = options
    else
      url = options[:url] || determine_redis_provider || 'redis://localhost:6379/0'
      driver = options[:driver] || 'ruby'
      namespace = options[:namespace] || 'merry_go_round'

      client = Redis.connect(url: url, driver: driver)
    end

    @@redis = Redis::Namespace.new(namespace, redis: client)
  end

  def self.granularities
    GRANULARITIES
  end

private

  def self.determine_redis_provider
    return ENV['REDISTOGO_URL'] if ENV['REDISTOGO_URL']
    provider = ENV['REDIS_PROVIDER'] || 'REDIS_URL'
    ENV[provider]
  end
end

require 'merry_go_round/version'
require 'merry_go_round/utils'
require 'merry_go_round/aggregator'
require 'merry_go_round/entry'
require 'merry_go_round/aggregation'
require 'merry_go_round/query'
require 'merry_go_round/web'
require 'merry_go_round/railtie' if defined? Rails

